<?php
namespace Folha\Entities;

use HCM\Util\DateUtil;
use Importacao\Logic\Integracao\ProjetoImportacao;

class MigMovimentacao extends Base\MigMovimentacao
{

    /**
     * @param $nrmovimentacao
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setNrmovimentacao ($nrmovimentacao) {
        $nrmovimentacao = \Importacao\Logic\Validacoes::verificaChaveDuplicada('GPE_MOVIMENTACAO', 'NRMOVIMENTACAO', ProjetoImportacao::getNrorg(), $nrmovimentacao);
        $this->nrmovimentacao = $nrmovimentacao;
    }

    /**
     * @param $nrvinculom
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setNrvinculom ($nrvinculom) {
        $nrvinculom = \Importacao\Logic\Validacoes::verificaChaveMae('GPE_VINCULOM', 'NRVINCULOM', ProjetoImportacao::getNrorg(), $nrvinculom);
        $this->nrvinculom = $nrvinculom;
    }

    /**
     * @param $nrtipotransfer
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setNrtipotransfer ($nrtipotransfer) {

        if ($nrtipotransfer != "") {
            $nrorg = ProjetoImportacao::getNrorgPadrao();
            $tipotransfer = \Folha\Entities\GpeTipotransfer::findOneByNrtipotransferAndNrorg($nrtipotransfer, $nrorg);

            if (is_object($tipotransfer)) {
                $this->nrtipotransfer = $nrtipotransfer;
            } else {
                throw new \Exception("O código do Tipo de Tranferência '$nrtipotransfer' não foi encontrado.");
            }
        } else {
            throw new \Exception("Campo Obrigatório: Tipo de Tranferência.");
        }
    }

    /**
     * @param $nrestruturam
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setNrestruturam ($nrestruturam) {
        $nrestruturam = \Importacao\Logic\Validacoes::verificaChaveMae('ESTRUTURAM', 'NRESTRUTURAM', ProjetoImportacao::getNrorg(), $nrestruturam);
        $this->nrestruturam = $nrestruturam;
    }

    /**
     * @param $nrtipoestrutura
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setNrtipoestrutura ($nrtipoestrutura) {
        if ($nrtipoestrutura != "") {
            $nrorg = ProjetoImportacao::getNrorgPadrao();
            $tipoestrutura = \Geral\Entities\Tipoestrutura::findOneByNrtipoestruturaAndNrorg($nrtipoestrutura, $nrorg);

            if (is_object($tipoestrutura)) {
                $this->nrtipoestrutura = $nrtipoestrutura;
            } else {
                throw new \Exception("O código do Tipo de Estrutura '$nrtipoestrutura' não foi encontrado.");
            }
        } else {
            throw new \Exception("Campo Obrigatório: Tipo de Estrutura.");
        }
    }

    /**
     * @param $dtmovimentretro
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setDtmovimentretro ($dtmovimentretro) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtmovimentretro)) {
            if (!empty($dtmovimentretro)) {
                throw new \Exception("Dt. Mov. Retroativa: Data '$dtmovimentretro' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtmovimentretro instanceof \DateTime)) {
            $dtmovimentretro = DateUtil::getDataDeString($dtmovimentretro);
        }

        $this->dtmovimentretro = $dtmovimentretro;
    }

    /**
     * @param $dtinimoviment
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setDtinimoviment ($dtinimoviment) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinimoviment)) {
            throw new \Exception("Campo Obrigatório - Dt. Início Movimentação: Data '$dtinimoviment' com formato inválido ou em branco.");
        }

        if (!($dtinimoviment instanceof \DateTime)) {
            $dtinimoviment = DateUtil::getDataDeString($dtinimoviment);
        }

        $this->dtinimoviment = $dtinimoviment;
    }

    /**
     * @param $dtfimmoviment
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setDtfimmoviment ($dtfimmoviment) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtfimmoviment)) {
            if (!empty($dtfimmoviment)) {
                throw new \Exception("Dt. Fim Movimentação: Data '$dtfimmoviment' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtfimmoviment instanceof \DateTime)) {
            $dtfimmoviment = DateUtil::getDataDeString($dtfimmoviment);
        }

        $this->dtfimmoviment = $dtfimmoviment;
    }

    /**
     * @param $dtultatu
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setDtultatu ($dtultatu) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtultatu)) {
            throw new \Exception("Data '$dtultatu' com formato inválido ou em branco.");
        }

        if (!($dtultatu instanceof \DateTime)) {
            $dtultatu = DateUtil::getDataDeString($dtultatu);
        }

        $this->dtultatu = $dtultatu;
    }

    /**
     * @param $dtinclusao
     * @throws \Exception
     * @return \Folha\Entities\MigMovimentacao
     */
    public function setDtinclusao ($dtinclusao) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinclusao)) {
            throw new \Exception("Data '$dtinclusao' com formato inválido ou em branco.");
        }

        if (!($dtinclusao instanceof \DateTime)) {
            $dtinclusao = DateUtil::getDataDeString($dtinclusao);
        }

        $this->dtinclusao = $dtinclusao;
    }

}